#!/usr/bin/env perl

# custom data xtract
# 2002, Michael Feig, Brooks group, TSRI
#
# extract ensemble of structures from REX archives based on condition
# cjh, 2004

sub usage {
  printf STDERR "usage:    extractEns.pl [options] \n";
  printf STDERR "options:  [-pdb template] -dir workdir \n";
  printf STDERR "          [-inx from[:to]] [-step n] -cond from[:to] \n";
  printf STDERR "          [-savedir dir] [-ens tag] -merge \n";
  exit 1;
}

use vars qw ( $perllibdir );

BEGIN {
  $perllibdir="$ENV{MMTSBDIR}/perl" if (defined $ENV{MMTSBDIR});
  ($perllibdir=$0)=~s/[^\/]+$// if (!defined $perllibdir);
}

use lib $perllibdir;
use strict;

use GenUtil;
use Molecule;
use Ensemble;
use ReXServer;

my $pdbtemplate;
my $fname=sprintf("tmp.$$.pdb");
my $dir=".";
my $from=1;
my $to;
my $condfrom=0;
my $condto=0;
my $step=1;
my $savedir="ens";
my $savetag="rex";
my $enstag;
my $merge=0;

while ( $#ARGV >= 0 ) {
  if ($ARGV[0] eq "-inx") {
    shift @ARGV;
    my @f=split(/:/,shift @ARGV);
    $from=$f[0];
    $to=$f[1] if(defined $f[1]);
  } elsif ($ARGV[0] eq "-step") {
    shift @ARGV;
    $step=shift @ARGV;
  } elsif ($ARGV[0] eq "-cond") {
    shift @ARGV;
    my @f=split(/:/,shift @ARGV);
    $condfrom=$f[0];
    $condto=(defined $f[1])?$f[1]:$condfrom;
  } elsif ($ARGV[0] eq "-pdb") {
    shift @ARGV;
    $pdbtemplate=shift @ARGV;
  } elsif ($ARGV[0] eq "-ens") {
    shift @ARGV;
    $savetag=shift @ARGV;
  } elsif ($ARGV[0] eq "-savedir") {
    shift @ARGV;
    $savedir=shift @ARGV;
  } elsif ($ARGV[0] eq "-dir") {
    shift @ARGV;
    $dir=shift @ARGV;
  } elsif ($ARGV[0] eq "-merge") {
    shift @ARGV;
    $merge=1;
  } elsif ($ARGV[0] eq "-help" || $ARGV[0] eq "-h") {
    &usage();
  } elsif ($#ARGV>=0) {
    print STDERR "Unknown option $ARGV[0]\n";
    shift @ARGV;
  }
}

my $rex=ReXServer->new(0,$dir);
my $condfile=$dir."/rexserver.cond";
$rex->setup($condfile);
$rex->readData();

my @e=();
my @at=();
my %cond;
if($merge) {
    my $tmp=$savedir;
    for (my $i=0; $i<=$#{$rex->{cond}}; $i++) {
	$tmp.=$i if( $i>=$condfrom && $i<=$condto );
    }
    my $ens=Ensemble->new($savetag,$tmp);
    push(@e,$ens);
    push(@at,$ens->{par}->{runs}+1);
} else {    
    for (my $i=0; $i<=$#{$rex->{cond}}; $i++) {
	if( $i>=$condfrom && $i<=$condto ) {
	    my $ens=Ensemble->new($savetag,$savedir.$i);
	    $ens->readFileList();
	    push(@e,$ens);
	    push(@at,$ens->{par}->{runs}+1);
	    $cond{$i}=$#e;
	}
    }
}

my $hashinx=0;
my $max;
if(defined $to) {
    $max = (($rex->{trun}-$rex->{par}->{initruns})<$to)?
                ($rex->{trun}-$rex->{par}->{initruns}):$to;
} else {
    $max = $rex->{trun}-$rex->{par}->{initruns};
} 
for (my $ir=$from; $ir<=$max; $ir+=$step) {
  foreach my $cid (@{$rex->{clientid}}) { 
    my $cdat=$rex->getClientData($cid,$ir);
    my $condinx=$cdat->{cond}->{inx};
    my $ener=$cdat->{ener};
    my $rmsd=$cdat->{rmsd};

    if ($condinx>=$condfrom && $condinx<=$condto ) {

	printf STDERR "run %d client %s ener %f rmsd %f temp %f\n",
                       $ir,$cid,$ener,$rmsd,$cdat->{cond}->{temp};
    
	if( -r "$dir/$cid/prod.coor.archive" ) {
	    my $arfile="$dir/$cid/prod.coor.archive";
	    my $pdbfile=(defined $pdbtemplate)?$pdbtemplate:"$dir/$cid/final.pdb";

	    my $data=&GenUtil::readArchiveFile($arfile,$ir);

	    open OUT,">$fname";    
	    my $start=0;
	    my $templ=&GenUtil::getInputFile($pdbfile);
	    while (<$templ>) {
		if (/ATOM/) {
		    substr($_,30,24)=substr($data,$start,24);
		    $start+=24;
		}
		print OUT $_;
	    }
	    close $templ;
	    close OUT;
	} else {
	    my $dir1 = $ir%100;
	    my $dir2 = int($ir/100);
	    system("zcat $cid/prod/$dir2/$dir1/final.pdb.gz > $fname");
	}

	my $mol=Molecule::new();
	$mol->readPDB("$fname");
	$hashinx=$cond{$condinx} if(!$merge);	    
	$e[$hashinx]->setFileList($at[$hashinx],sprintf("$dir.$ir.$cid"));
	$e[$hashinx]->checkinPDB($at[$hashinx],$mol);
	$e[$hashinx]->setProp("rexener",$at[$hashinx],$ener) if (defined $ener);
	$e[$hashinx]->setProp("rexrmsd",$at[$hashinx],$rmsd) if (defined $rmsd);
	$at[$hashinx]++;
    }
  }
}

&GenUtil::remove($fname);

foreach my $te ( @e ) {
  $te->save();
}

exit 0;

